# -*- coding: utf-8 -*-
import openerp
import os
from subprocess import Popen, PIPE
from pysnmp.entity.rfc3413.oneliner import cmdgen
from pysnmp.proto import rfc1902
from openerp.osv import osv
from openerp import models, fields, api
import nmap
import smtplib
from datetime import datetime

"""
Declaración de los campos/variables que contienen los objetos del menú Dispositivos

Cada <dispositivo> incorpora una función interna para buscar MIBS (Walk)

rangomib: Campo que utilizará la función walk_mib para realizar un GETNEXT. Se introduce el rango de OIDS, la función se encarga de encontrar los MIBS que entren dentro del rango

        Ejemplo:
            Introducimos '1.3.6.1.2.1.1' en un router de ejemplo.
            La función encuentra todos los MIBS del dispositivo que se encuentren dentro del rango.
            En el caso del ejemplo: '1.3.6.1.2.1.1.1', '1.3.6.1.2.1.1.2', '1.3.6.1.2.1.1.3', '1.3.6.1.2.1.1.4', ...

    dispomib_id: Campo que crea una relación one2many con la lista de mibs. Asigna varios objetos del tipo MIB a un dispositivo.

"""

class lista_dispo(models.Model):

    _name = 'lista_dispo.listaip'

    name = fields.Char(string="Dispositivo: ")
    nombre = fields.Char(string="Nombre: ")

    version = fields.Selection((('v1', 'SNMPv1'), ('v2', 'SNMPv2'),
    ('v3', 'SNMPv3')), string="Versión SNMP: ")

    community = fields.Char("Community: ")

    rangomib = fields.Char("OID: ", default = "1.3.6")

    userv3 = fields.Char("Usuario SNMPv3: ")
    passv3 = fields.Char("Pass SNMPv3: ")

    dispomib_id = fields.One2many('lista_mib.listamib', 'dispo_id',
    string="Lista MIBs")

    imagen = fields.Binary("Imagen", help="Seleccionar imagen")

    def walk_mib(self, cr, uid, ids, context=None):
    #Funcion GETNEXT que se ejecuta al pulsar el botón Buscar, en el Form
        ipversion = self.read(cr, uid, ids, ['version'], context=context)
        ipversion = ipversion[0]['version']

        #Para SNMPv2
        if(str(ipversion) == 'v2'):

            cmdGen = cmdgen.CommandGenerator()

            ipdis = self.read(cr, uid, ids, ['name'], context=context)
            ips = ipdis[0]['name']

            oidis = self.read(cr, uid, ids, ['rangomib'], context=context)
            dispoids = oidis[0]['rangomib']

            comundis = self.read(cr, uid, ids, ['community'], context=context)
            comun = comundis[0]['community']

            errorIndication, errorStatus, errorIndex, varBindTable = cmdGen.nextCmd(
                cmdgen.CommunityData(comun),
                cmdgen.UdpTransportTarget((ips, 161)),
                str(dispoids),
                lookupNames=False, lookupValues=False
            )

            if errorIndication:
                print(errorIndication)
            else:
                if errorStatus:
                    print('%s at %s' % (
                        errorStatus.prettyPrint(),
                        errorIndex and varBindTable[-1][int(errorIndex) - 1] or '?'
                        )
                    )
                else:
                    tabla = varBindTable
                    obj = self.pool.get('lista_mib.listamib')
                    for record in self.browse(cr, uid, ids, context=context):
                        for fila in tabla:
                            for oids, valors in fila:

                                oid = oids.prettyPrint()
                                val = valors.prettyPrint()
                                obj.create(cr, uid, {'mib_oid': oid, 'value': val,
                                'dispo_id': record.id, 'community': comun},
                                context=None)
        #Para SNMPv1
        elif (str(ipversion) == 'v1'):

            cmdGen = cmdgen.CommandGenerator()

            ipdis = self.read(cr, uid, ids, ['name'], context=context)
            ips = ipdis[0]['name']

            oidis = self.read(cr, uid, ids, ['rangomib'], context=context)
            dispoids = oidis[0]['rangomib']

            comundis = self.read(cr, uid, ids, ['community'], context=context)
            comun = comundis[0]['community']

            errorIndication, errorStatus, errorIndex, varBindTable = cmdGen.nextCmd(
                cmdgen.CommunityData(comun),
                cmdgen.UdpTransportTarget((ips, 161)),
                str(dispoids),
                lookupNames=False, lookupValues=False
            )

            if errorIndication:
                print(errorIndication)
            else:
                if errorStatus:
                    print('%s at %s' % (
                        errorStatus.prettyPrint(),
                        errorIndex and varBindTable[-1][int(errorIndex) - 1] or '?'
                        )
                    )
                else:
                    tabla = varBindTable
                    obj = self.pool.get('lista_mib.listamib')
                    for record in self.browse(cr, uid, ids, context=context):
                        for fila in tabla:
                            for oids, valors in fila:

                                oid = oids.prettyPrint()
                                val = valors.prettyPrint()
                                obj.create(cr, uid, {'mib_oid': oid, 'value': val,
                                'dispo_id': record.id, 'community': comun},
                                context=None)
        #Para SNMPv3
        elif (str(ipversion) == 'v3'):

            ipdis = self.read(cr, uid, ids, ['name'], context=context)
            ips = ipdis[0]['name']

            comundis = self.read(cr, uid, ids, ['community'], context=context)
            comun = comundis[0]['community']

            oidis = self.read(cr, uid, ids, ['rangomib'], context=context)
            dispoids = oidis[0]['rangomib']

            userdis = self.read(cr, uid, ids, ['userv3'], context=context)
            user = userdis[0]['userv3']

            passwdis = self.read(cr, uid, ids, ['passv3'], context=context)
            passw = passwdis[0]['passv3']

            cmdGen = cmdgen.CommandGenerator()

            errorIndication, errorStatus, errorIndex, varBinds = cmdGen.getCmd(
                cmdgen.CommunityData(comun),
                cmdgen.UsmUserData(str(user), str(passw)),
                cmdgen.UdpTransportTarget((ips, 161)),
                str(dispoids)
                #cmdgen.MibVariable('IP-MIB', 'ipAdEntAddr', ips)
            )

            if errorIndication:
                print(errorIndication)
            else:
                if errorStatus:
                    print('%s at %s' % (
                        errorStatus.prettyPrint(),
                        errorIndex and varBinds[int(errorIndex)-1] or '?'
                        )
                    )
                else:

                    tabla = varBinds
                    obj = self.pool.get('lista_mib.listamib')
                    for record in self.browse(cr, uid, ids, context=context):
                        for oids, vals in tabla:

                                oid = oids.prettyPrint()
                                val = vals.prettyPrint()

                                obj.create(cr, uid, {'mib_oid': oid, 'value': val,
                                'dispo_id': record.id, 'community': comun},
                                context=None)
lista_dispo()

"""
Declaración de los campos/variables que contienen los objetos del menú MIBS

Incorpora:
Activar un CRON que consulta el valor de un MIB cada X minutos y los guarda en el Log de registros
Gráfica de los últimos registros.
Condiciones para que lance un aviso, y el tipo de aviso (mail, set value, terminal command)


Aviso mail:



"""

class lista_mib(models.Model):

    _name = 'lista_mib.listamib'

    mib_oid = fields.Char(string="OID")
    name = fields.Char(string="Nombre MIB: ")
    value = fields.Char(string="Valor")
    refresh_time = fields.Float(string="Tiempo comprobacion")
    community = fields.Char(string="commu")

    alerta_mail = fields.Boolean()
    alerta_consola = fields.Boolean()
    alerta_set = fields.Boolean()
    user = fields.Many2one('res.users', 'Usuario', select=True)

    valor_set = fields.Char()

    temp_activo = fields.Boolean()

    dispo_id = fields.Many2one('lista_dispo.listaip', ondelete='cascade',
    string="Dispositivo")

    host_id = fields.Many2one('hosts.host', ondelete='cascade')
    mib_id = fields.Many2one('lista_mib.listamib', ondelete='cascade')

    condiciones_id = fields.One2many('condiciones.condicion', 'condicion_id',
    string="Condiciones")

    body = fields.Text()

    imagen = fields.Binary("Imagen", help="Seleccionar imagen")

    email = fields.Boolean()
    consola = fields.Boolean()
    set_val = fields.Boolean()

    comando_terminal = fields.Text()

    def valor_actual(self, cr, uid, ids, context=None):

        """mib_id = self.read(cr, uid, ids, ['mib_id'], context=context)
        mib_id = mib_id[0]['mib_id']
        
        mib_id = mib_id[1]"""

        obj_mibs = self.pool.get('lista_mib.listamib')

        """buscamib_id = obj_mibs.search(cr, uid, [('name', '=', mib_id)],
        context=context)"""

        comundis = obj_mibs.read(cr, uid, ids, ['community'], context=context)
        comundis = comundis[0]['community']

        dispoip = obj_mibs.read(cr, uid, ids, ['dispo_id'], context=context)
        dispoip = dispoip[0]['dispo_id']
        dispoip = dispoip[1]

        oidis = obj_mibs.read(cr, uid, ids, ['mib_oid'], context=context)
        oidis = oidis[0]['mib_oid']

        #raise osv.except_osv(('Error'), dispoip[1])
        cmdGen = cmdgen.CommandGenerator()

        errorIndication, errorStatus, errorIndex, varBinds = cmdGen.getCmd(
        cmdgen.CommunityData(comundis),
        cmdgen.UdpTransportTarget((dispoip, 161)),
        str(oidis),
        lookupNames=False, lookupValues=False
        )

        mib = varBinds
        for oids, valors in mib:

            oid = oids.prettyPrint()
            val = valors.prettyPrint()

        obj_mibs.write(cr, uid, ids, {'value': val},
        context=None)

    def crea_grafico(self, cr, uid, ids, context=None):

        records_usados = self.pool.get('wizard_grafico.oid')
        ids_usadas = records_usados.search(cr, uid, [])

        records_usados.unlink(cr, uid, ids_usadas, context=None)

        dispo_actual = self.read(cr, uid, ids, ['dispo_id'], context=context)
        dispo_actual = dispo_actual[0]['dispo_id']

        oid_dispo = self.read(cr, uid, ids, ['mib_oid'], context=context)
        oid_dispo = oid_dispo[0]['mib_oid']

        name_dispo = self.read(cr, uid, ids, ['name'], context=context)
        name_dispo = name_dispo[0]['name']

        obj_log = self.pool.get('log_mib.logmib')

        ids = obj_log.search(cr, uid, [('oid', '=', oid_dispo),
        ('dispoid', '=', str(dispo_actual[1])), ('nombre', '=', str(name_dispo))],
        context=context)

        obj_wizard = self.pool.get('wizard_grafico.oid')

        if len(ids) <= 1440:
            for record in obj_log.browse(cr, uid, ids, context=context):

                obj_wizard.create(cr, uid, {'nombredispo': record.nombredispo,
                                            'value': record.value,
                                            'dispoid': record.dispoid},
                                            context=None)
            #raise osv.except_osv(('Error'), cnt)
        else:
            i = len(ids) - 1440
            n = 0
            for record in obj_log.browse(cr, uid, ids, context=context):
                if n < i:
                    n = n + 1
                else:
                    obj_wizard.create(cr, uid, {'nombredispo': record.nombredispo,
                                            'value': record.value,
                                            'dispoid': record.dispoid},
                                            context=None)


        return {
           'name': 'Grafico',
           'view_type': 'graph',
           'view_mode': 'graph',
           'res_model': 'wizard_grafico.oid',
           'type': 'ir.actions.act_window',
           'target': 'new'
           }

    def copiar_log(self, cr, uid, ids, context=None):

        reftimedis = self.read(cr, uid, ids, ['refresh_time'], context=context)
        reftime = reftimedis[0]['refresh_time']

        nombremib = self.read(cr, uid, ids, ['name'], context=context)
        nombremib = nombremib[0]['name']

        self.pool.get('ir.cron').create(cr, uid, {
            'name': str(nombremib),
            'interval_number': reftime,
            'interval_type': 'minutes',
            'numbercall': '-1',
            'doall': True,
            'user_id': uid,
            'model': 'lista_mib.listamib',
            'function': 'get_mibs',
            'args': [ids]
            }, context=context)

        cronos = self.pool.get('ir.cron')

        crono_id = cronos.search(cr, uid, [('name', '=', nombremib)],
        context=context)

        crono_activo = cronos.read(cr, uid, crono_id, ['active'],
        context=context)
        crono_activo = crono_activo[0]['active']

        self.write(cr, uid, ids, {'temp_activo': bool(crono_activo)},
        context=None)

    def get_mibs(self, cr, uid, ids, context=None):

        dispo = self.read(cr, uid, ids, ['dispo_id'], context=context)
        dispo = dispo[0]['dispo_id']
        dispo = dispo[1]

        oid_dispo = self.read(cr, uid, ids, ['mib_oid'], context=context)
        oid_dispo = oid_dispo[0]['mib_oid']

        comu = self.read(cr, uid, ids, ['community'], context=context)
        comu = comu[0]['community']

        nombre = self.read(cr, uid, ids, ['name'], context=context)
        nombre = nombre[0]['name']

        body = self.read(cr, uid, ids, ['body'], context=context)
        body = body[0]['body']

        alerta_mail = self.read(cr, uid, ids, ['alerta_mail'], context=context)
        alerta_mail = alerta_mail[0]['alerta_mail']

        alerta_consola = self.read(cr, uid, ids, ['alerta_consola'],
        context=context)
        alerta_consola = alerta_consola[0]['alerta_consola']

        alerta_set = self.read(cr, uid, ids, ['alerta_set'], context=context)
        alerta_set = alerta_set[0]['alerta_set']

        usermail = self.read(cr, uid, ids, ['user'], context=context)
        usermail = usermail[0]['user']
        user_name = usermail[1]

        usuarios = self.pool.get('res.users')
        usuario_id = usuarios.search(cr, uid, [('name', '=', user_name)],
        context=context)
        usuario_mail = usuarios.read(cr, uid, usuario_id, ['login'],
        context=context)
        usuario_mail = usuario_mail[0]['login']

        comando = self.read(cr, uid, ids, ['comando_terminal'], context=context)
        comando = comando[0]['comando_terminal']

        obj_log = self.pool.get('log_mib.logmib')

        cmdGen = cmdgen.CommandGenerator()

        errorIndication, errorStatus, errorIndex, varBinds = cmdGen.getCmd(
            cmdgen.CommunityData(comu),
            cmdgen.UdpTransportTarget((dispo, 161)),
            str(oid_dispo),
            lookupNames=False, lookupValues=False
            )

        tabla = varBinds
        for oids, valors in tabla:

                oid = oids.prettyPrint()
                val = valors.prettyPrint()

        obj_log.create(cr, uid, {'oid': oid,
                                'value': val,
                                'dispoid': dispo,
                                'nombre': nombre,
                                'nombredispo': datetime.utcnow()},
                                context=None)

        aviso_def = False

        lista_condiciones = self.read(cr, uid, ids,
        ['condiciones_id'], context=context)
        lista_condiciones = lista_condiciones[0]['condiciones_id']

        obj_condiciones = self.pool.get('condiciones.condicion')

        condicio = []
        marca = []
        avisar = []

        for cond in obj_condiciones.browse(cr, uid, lista_condiciones,
        context=context):
            condicio.append(cond)
            marca.append(False)
            avisar.append(False)

        n = 0

        for condicion in obj_condiciones.browse(cr, uid, lista_condiciones,
        context=context):

            if n == 0:
                if condicion.compara == 'mayor':
                    if float(val) > condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'menor':
                    if float(val) < condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'mayorigual':
                    if float(val) >= condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'menorigual':
                    if float(val) <= condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'igual':
                    if float(val) == condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'distinto':
                    if float(val) != condicion.valor:
                        marca[n] = True
                if condicio[n].o_log != 'y' and condicio[n].o_log != 'o':
                    avisar[n] = marca[n]
                if n == len(condicio) - 1:
                    avisar[n] = marca[n]
                else:
                    n = n + 1
            else:
                if condicion.compara == 'mayor':
                    if float(val) > condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'menor':
                    if float(val) < condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'mayorigual':
                    if float(val) >= condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'menorigual':
                    if float(val) <= condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'igual':
                    if float(val) == condicion.valor:
                        marca[n] = True
                elif condicion.compara == 'distinto':
                    if float(val) != condicion.valor:
                        marca[n] = True
                if condicio[n - 1].o_log == 'o':
                    if marca[n - 1] is True:
                        avisar[n] = True
                elif condicio[n - 1].o_log == 'y':
                    if marca[n - 1] is False and marca[n] is True:
                        marca[n] = False
                    elif marca[n - 1] is True and marca[n] is True:
                        avisar[n] = True
                    else:
                        avisar[n] = False
                elif condicio[n - 1].o_log != 'y' and condicio[n - 1].o_log != 'o' and condicio[n].o_log != 'y' and condicio[n].o_log != 'o':
                    avisar[n] = marca[n]

                n = n + 1

            for i in avisar:
                if i is True:
                    aviso_def = True
            if(alerta_mail):

                id_host = self.read(cr, uid, ids, ['host_id'], context=context)
                id_host = id_host[0]['host_id']
                id_host = id_host[1]

                hosts = self.pool.get('hosts.host')
                hostid = hosts.search(cr, uid, [('name', '=', id_host)],
                context=context)

                dirhost = hosts.read(cr, uid, hostid, ['host_smtp'], context=context)
                dirhost = dirhost[0]['host_smtp']

                porthost = hosts.read(cr, uid, hostid, ['puerto_smtp'], context=context)
                porthost = porthost[0]['puerto_smtp']

                usersmtp = hosts.read(cr, uid, hostid, ['usuario_smtp'], context=context)
                usersmtp = usersmtp[0]['usuario_smtp']

                contrasmtp = hosts.read(cr, uid, hostid, ['contra_smtp'], context=context)
                contrasmtp = contrasmtp[0]['contra_smtp']

                if(aviso_def):
                    smtpObj = smtplib.SMTP(host=dirhost,
                    port=str(porthost))
                    smtpObj.ehlo()
                    smtpObj.starttls()
                    smtpObj.ehlo()
                    #raise osv.except_osv(('Error'), usersmtp+contrasmtp)
                    smtpObj.login(user=str(usersmtp),
                    password=str(contrasmtp))
                    mensaje = 'Subject: %s\n\n%s' % ('ALERTA', body)
                    smtpObj.sendmail(usersmtp, usuario_mail,
                    mensaje + "\nValor:" + val)

            if(alerta_consola):

                comando = self.read(cr, uid, ids, ['comando_terminal'], context=context)
                comando = comando[0]['comando_terminal']

                proc=Popen(comando, stdin=PIPE, shell=True)
                #proc.communicate('password')
                #proc.communicate('123')
                

            if(alerta_set):

                valor_nuevo = self.read(cr, uid, ids, ['valor_set'], context=context)
                valor_nuevo = valor_nuevo[0]['valor_set']

                mib_id = self.read(cr, uid, ids, ['mib_id'], context=context)
                mib_id = mib_id[0]['mib_id']
                mib_id = mib_id[1]

                obj_mibs = self.pool.get('lista_mib.listamib')

                buscamib_id = obj_mibs.search(cr, uid, [('name', '=', mib_id)],
                context=context)

                comundis = obj_mibs.read(cr, uid, buscamib_id, ['community'], context=context)
                comundis = comundis[0]['community']

                dispoip = obj_mibs.read(cr, uid, buscamib_id, ['dispo_id'], context=context)
                dispoip = dispoip[0]['dispo_id']
                dispoip = dispoip[1]

                oidis = obj_mibs.read(cr, uid, buscamib_id, ['mib_oid'], context=context)
                oidis = oidis[0]['mib_oid']

                #raise osv.except_osv(('Error'), dispoip[1])
                cmdGen = cmdgen.CommandGenerator()

                errorIndication, errorStatus, errorIndex, varBinds = cmdGen.setCmd(
                cmdgen.CommunityData(comundis, mpModel=0),
                cmdgen.UdpTransportTarget((dispoip, 161)),
                (str(oidis), valor_nuevo)
                )
                # Check for errors and print out results
                if errorIndication:
                    print(errorIndication)
                else:
                    if errorStatus:
                        print('%s at %s' % (
                        errorStatus.prettyPrint(),
                        errorIndex and varBinds[int(errorIndex)-1] or '?'
                    )
                )

    def eliminar_cron(self, cr, uid, ids, context=None):

        cronos = self.pool.get('ir.cron')

        nom = self.read(cr, uid, ids, ['name'], context=context)
        nom = nom[0]['name']

        #raise osv.except_osv(('Error'), nom)

        crono_id = cronos.search(cr, uid, [('name', '=', nom)],
        context=context)

        cronos.unlink(cr, uid, crono_id, context=None)

        self.write(cr, uid, ids, {'temp_activo': False},
        context=None)

lista_mib()


class log_mib(models.Model):

    _name = 'log_mib.logmib'

    dispoid = fields.Char(string="Dispositivo")
    oid = fields.Char(string="OID")
    value = fields.Float(string="Valor")
    nombredispo = fields.Char("Registros")
    nombre = fields.Char()

log_mib()


class oid_wizard(models.Model):

    _name = 'wizard_grafico.oid'

    nombredispo = fields.Char()
    dispoid = fields.Char()
    value = fields.Float()

oid_wizard()


class condiciones(models.Model):

    _name = 'condiciones.condicion'

    compara = fields.Selection((('mayor', 'Mayor'),
                            ('menor', 'Menor'),
                            ('mayorigual', 'Mayor o igual'),
                            ('menorigual', 'Menor o igual'),
                            ('igual', 'Igual'),
                            ('distinto', 'Distinto')))

    valor = fields.Float(string="Valor")

    o_log = fields.Selection((('y', 'Y'),
                            ('o', 'O')))

    condicion_id = fields.Many2one('lista_mib.listamib', ondelete='cascade',
    string="Condicion")

condiciones()


class host_smtp(models.Model):

    _name = 'hosts.host'

    name = fields.Char()

    host_smtp = fields.Char()
    puerto_smtp = fields.Char()

    usuario_smtp = fields.Char()
    contra_smtp = fields.Char()
